/*
 * (c) Peter Mueller, 2007
 *
 * All rights reserved. Reproduction, modification,
 * use or disclosure to third parties without express
 * authority is forbidden.
 *
 * $Id: counter.c 57 2006-08-29 20:23:11Z pmueller $
 *
 * Low level I2c bus interface functions for MSP430 in software    
 */

#include <msp430.h>
#include <intrinsics.h>
#include <stdint.h>
#include "i2c.h"


#define SDA       0x4  // Serial Data Line
#define SCL       0x2  // Serial Clock Line

#define I2CWAIT 

#define NO_I2C_ACK 0
#define OK_I2C_ACK 1

/*************************/

static void SetLowSDA()
{
   P2DIR |= SDA;  // set SDA to output pin. 
   _nop();
}
static void SetHighSDA()
{
   P2DIR &= ~SDA; // set SDA to input pin (resistor keeps it high)
   _nop();
}

void SetLowSCL()
{
   P2OUT &= ~SCL;
   _nop();
}
void SetHighSCL()
{
   P2OUT |= SCL;
   _nop();
}

void i2cLowPower(void){
   P2DIR &= ~SDA;    // set to in
   P2DIR &= ~SCL;    // set to in

}

/*************************/
/* To initialize the bus */
/*************************/
void i2cInit(void)
{
   P2SEL &=  ~SDA;
   P2SEL &=  ~SCL;   
  
   P2OUT |= SCL;   
   P2OUT &= ~SDA;
   
   P2DIR |= SCL;     // set to out
   P2DIR &= ~SDA;    // set to in

   SetHighSCL();
   SetLowSDA(); 
   SetHighSDA();
}

/*************************/
/* Start Data Transfer   */
/*************************/
void i2cStart()
{

  SetHighSCL();
  SetHighSDA();

  SetHighSCL();
  SetLowSDA();

  SetLowSCL();
  SetHighSDA();
}
/*************************/
/* Stop  Transfer        */
/*************************/
void i2cStop()
{
  SetLowSCL();
  SetLowSDA();

  SetHighSCL();
  SetLowSDA();

  SetHighSCL();
  SetHighSDA();
}
/*************************/
/* Write  Transfer       */
/*************************/
int i2cWrite(char a)
{
  int i;
  int return_ack;

  for (i=0; i<8; i++){
     SetLowSCL();
     if (a & 0x80)          
       SetHighSDA();
     else
       SetLowSDA();

     SetHighSCL();
     a <<= 1;
  }
  SetLowSCL();

  /* ack Read */
  SetHighSDA();

  SetHighSCL();

  if (P2IN & SDA)         
   return_ack = NO_I2C_ACK;      
  else
   return_ack = OK_I2C_ACK;
  SetLowSCL();
  return(return_ack);
}
/*************************/
/* Read  Transfer        */
/*************************/
char  i2cRead(int ack)
{
  int i;
  char caracter = 0x00;

  SetLowSCL();
  SetHighSDA();

  for (i=0; i<8; i++){
     caracter = caracter << 1;
     SetHighSCL();
     if (P2IN & SDA)
       caracter = caracter  + 1;
     SetLowSCL();
  }

  if (ack){                          
     SetLowSDA();
  }
  SetHighSCL();
  SetLowSCL();

  return (caracter);
}

