/*
 * (c) Peter Mueller, 2007
 *
 * All rights reserved. Reproduction, modification,
 * use or disclosure to third parties without express
 * authority is forbidden.
 *
 * $Id: counter.c 57 2006-08-29 20:23:11Z pmueller $
 *
 */

#include <stdint.h>
#include "i2c.h"
#include "tmp100.h"

#define TEMPERATURE_REGISTER 0
#define CONFIGURATION_REGISTER 1

#define WRITE_TO_DEVICE 0x90
#define READ_FROM_DEVICE 0x91

#define CONFIG_DATA 0xe1 // One shot, 12bits, low-power mode

extern void delay(unsigned int d);


//
// Start conversion in TMP100 and read back temperature
// after the TMP100 specified delay time
// Preconditions:
//   - port pins initialized correctly
//
//  Return
//    temperature
void startTemperature(void){

  // start distance measurement
  i2cStart();
  i2cWrite(WRITE_TO_DEVICE);
  i2cWrite(CONFIGURATION_REGISTER); // low byte of address -> command register
  i2cWrite(CONFIG_DATA); // data -> ranging in centimeters
  i2cStop();

  // about 400ms  
  //delay(0x8fff);
}
  
uint16_t getTemperature(void){

  unsigned char high_temp;
  unsigned char low_temp;
  unsigned short raw_temperature;
  unsigned long temperature;

  // readback temp
  i2cStart();
  i2cWrite(WRITE_TO_DEVICE);
  i2cWrite(TEMPERATURE_REGISTER);
  i2cStart();
  i2cWrite(READ_FROM_DEVICE);
  high_temp = i2cRead(1);
  low_temp = i2cRead(0);
  
  i2cStop();
  
  // Return temperature
  // high byte=higher 8 bits of the temperature value
  // low byte=depending of resolution bit 9...12. Lower nibble always unused
  raw_temperature = ((((unsigned short)high_temp)<<8) + low_temp)>>4;
  
  // scale: 80C = 1280 (see tab. 5 in the temp100 data sheet)
  // factor 100 used to represent temp as fixedpoint: xx.yy -> xxyy
  temperature = ((unsigned long)raw_temperature*100*80)/1280;
    
  return (uint16_t)temperature;
}
